<?php
	
GFForms::include_feed_addon_framework();

class GFleadsquared extends GFFeedAddOn {
	
	protected $_version = GF_LeadSquared_VERSION;
	protected $_min_gravityforms_version = '1.9.10.16';
	protected $_slug = 'gravityformsleadsquared';
	protected $_path = 'gravityformsleadsquared/leadsquared.php';
	protected $_full_path = __FILE__;
	protected $_url = 'http://www.leadsquared.com';
	protected $_title = 'Gravity Forms LeadSquared Add-On';
	protected $_short_title = 'LeadSquared';
	protected $api = null;
	protected $fields_transient_name = 'gform_leadsquared_fields';
	private static $_instance = null;

	/* Permissions */
	protected $_capabilities_settings_page = 'gravityforms_leadsquared';
	protected $_capabilities_form_settings = 'gravityforms_leadsquared';
	protected $_capabilities_uninstall = 'gravityforms_leadsquared_uninstall';

	/* Members plugin integration */
	protected $_capabilities = array( 'gravityforms_leadsquared', 'gravityforms_leadsquared_uninstall' );
	
	/**
	 * Get instance of this class.
	 * 
	 * @access public
	 * @static
	 * @return GFleadsquared
	 */
	public static function get_instance() {
		
		if ( self::$_instance == null ) {
			self::$_instance = new self;
		}

		return self::$_instance;
		
	}

	/**
	 * Register needed styles.
	 * 
	 * @access public
	 * @return array $styles
	 */
	public function styles() {
		
		$styles = array(
			array(
				'handle'  => 'gform_leadsquared_form_settings_css',
				'src'     => $this->get_base_url() . '/css/form_settings.css',
				'version' => $this->_version,
				'enqueue' => array(
					array( 'admin_page' => array( 'form_settings' ) ),
				)
			)
		);
		
		return array_merge( parent::styles(), $styles );
		
	}
	/**
	 * Setup plugin settings fields.
	 * 
	 * @access public
	 * @return array
	 */
	public function plugin_settings_fields() {
		
		$description  = '<p>';
		$description .= sprintf(
			esc_html__( 'Capture your contact data from Gravity forms into LeadSquared', 'gravityformsleadsquared' ));
		$description .= '</p>';
						
		return array(
			array(
				'title'       => '',
				'description' => $description,
				'fields'      => array(
					array(
						'name'              => 'apidomain',
						'label'             => esc_html__( 'API host', 'gravityformsleadsquared' ),
						'type'              => 'text',
						'class'             => 'medium',
						'dependency'        => array( 'field' => 'authMode', 'values' => array( '', 'email' ) ),
						'feedback_callback' => array( $this, 'plugin_settings_email_feedback' )
					),
					array(
						'name'              => 'AccessKey',
						'label'             => esc_html__( 'Access Key', 'gravityformsleadsquared' ),
						'type'              => 'text',
						'class'             => 'medium',
						'dependency'        => array( 'field' => 'authMode', 'values' => array( '', 'email' ) ),
						'feedback_callback' => array( $this, 'plugin_settings_email_feedback' )
					),
					array(
						'name'              => 'SecretKey',
						'label'             => esc_html__( 'Secret Key', 'gravityformsleadsquared' ),
						'type'              => 'text',
						'class'             => 'medium',
						'dependency'        => array( 'field' => 'authMode', 'values' => array( '', 'email' ) ),
						'feedback_callback' => array( $this, 'plugin_settings_email_feedback' )
					),
					array(
						'type'              => 'save',
						'messages'          => array(
							'success' => esc_html__( 'LeadSquared settings have been updated.', 'gravityformsleadsquared' )
						),
					),
				),
			),
		);
		
	}
	


	/**
	 * Get feedback for email address and password fields.
	 * 
	 * @access public
	 * @param string $value
	 * @param array $field
	 * @return bool|null
	 */
	public function plugin_settings_email_feedback( $value, $field ) {
		
		if ( rgblank( $value ) ) {
			return null;
		}
		
		return $this->initialize_api();
		
	}
	public function get_activity_code() {
		
		$meta_data = $this->api->get_activity_type();
		
		if ( ! empty( $meta_data ) ) {
			$fields = $meta_data;
		}
		foreach ( $fields as $field ) {

			/* Add field to field map. */
			if($field['Tags'] == '{Custom}'){
				$field_map[] = array(
					'label'      => $field['DisplayName'],
					'value'		 => $field['ActivityEvent']
				);
			}
		}		
		return $field_map; 	
	}

	/**
	 * Setup fields for feed settings.
	 * 
	 * @access public
	 * @return array
	 */
	public function feed_settings_fields() {
		
		$feed = ( $this->get_posted_settings() ) ? $this->get_posted_settings() : $this->get_current_feed();
		
		/* Build base fields array. */
		$base_fields = array(
			'title'  => '',
			'fields' => array(
				array(
					'name'           => 'feedName',
					'label'          => esc_html__( 'Feed Name', 'gravityformsleadsquared' ),
					'type'           => 'text',
					'required'       => true,
					'default_value'  => $this->get_default_feed_name(),
					'tooltip'        => '<h6>'. esc_html__( 'Name', 'gravityformsleadsquared' ) .'</h6>' . esc_html__( 'Enter a feed name to uniquely identify this setup.', 'gravityformsleadsquared' )
				),
				array(
					'name'           => 'action',
					'label'          => esc_html__( 'Action', 'gravityformsleadsquared' ),
					'required'       => true,
					'type'           => 'hidden',
					'default_value'  => 'lead'
				)
			)
		);
	
		/* Build lead fields array. */
		$lead_fields = array(
			'title'      => esc_html__( 'Lead Details', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'       => 'leadCustomFields',
					'label'      => 'LeadSquared Map',
					'type'       => 'dynamic_field_map',
					'field_map'  => $this->get_field_map_for_module( 'Leads', 'dynamic' ),
					'tooltip'    => '<h6>'. esc_html__( 'Map Fields', 'gravityformsleadsquared' ) .'</h6>' . esc_html__( 'Map LeadSquared fields to corresponding gravity from fields', 'gravityformsleadsquared' )
				),
				array(
					'name'       => 'OwnerId',
					'label'      => esc_html__( 'Lead Owner', 'gravityformsleadsquared' ),
					'type'       => 'select',
					'choices'    => $this->get_users_for_feed_setting()
				),
				array(
					'name'       => 'Source',
					'label'      => esc_html__( 'Lead Source', 'gravityformsleadsquared' ),
					'type'       => 'text',
					'default_value'  => 'Gravity Form'
				)
			)
		);
		/* Build task fields array. */
		$acvity_fields = array(
			'title'      => esc_html__( 'Acitvity Details', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'                => 'postActivity',
					'label'               => esc_html__( 'Post Activity', 'gravityformsleadsquared' ),
					'type'                => 'checkbox',
					'onclick'             => "jQuery(this).parents('form').submit();",
					'choices'             => array(
						array(
							'name'  => 'postActivity',
							'label' => sprintf( 
								esc_html__( 'Post Activity to %s', 'gravityformsleadsquared' ), 
								rgars( $feed, 'action' ) ? ucfirst( rgar( $feed, 'action' ) ) : ucfirst( rgars( $feed, 'meta/action' ) )
							),
						),
					)
				),
				array(
					'name'                => 'ActivityEvent',
					'label'               => esc_html__( 'Activity Event', 'gravityformsleadsquared' ),
					'type'                => 'select',
					'choices'             => $this->get_activity_code(),
					'dependency'          => array( 'field' => 'postActivity', 'values' => array( '1' ) )
				),
				array(
					'name'                => 'ActivityNote',
					'type'                => 'textarea',
					'class'               => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
					'label'               => esc_html__( 'Activity Note', 'gravityformsleadsquared' ),
					'dependency'          => array( 'field' => 'postActivity', 'values' => array( '1' ) )
				)
			)
		);
		
		/* different account. */
		$difac_fields = array(
			'title'      => esc_html__( 'Another Account', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'                => 'anotherAccount',
					'label'               => esc_html__( 'Account', 'gravityformsleadsquared' ),
					'type'                => 'checkbox',
					'onclick'             => "jQuery(this).parents('form').submit();",
					'choices'             => array(
						array(
							'name'  => 'anotherAccount',
							'label' => sprintf( 
								esc_html__( 'Post lead to different account', 'gravityformsleadsquared' ), 
								rgars( $feed, 'action' ) ? ucfirst( rgar( $feed, 'action' ) ) : ucfirst( rgars( $feed, 'meta/action' ) )
							),
						),
					)
				),
				array(
					'name'                => 'AccessKey',
					'label'               => esc_html__( 'Access Key', 'gravityformsleadsquared' ),
					'type'                => 'text',
					'class'               => 'medium',
					'dependency'          => array( 'field' => 'anotherAccount', 'values' => array( '1' ) )
				),
				array(
					'name'                => 'SecretKey',
					'label'               => esc_html__( 'Secret Key', 'gravityformsleadsquared' ),
					'type'                => 'text',
					'class'               => 'medium',
					'dependency'          => array( 'field' => 'anotherAccount', 'values' => array( '1' ) )
				),
				array(
					'name'                => 'ActivityEventCode',
					'label'               => esc_html__( 'Activity Event Code', 'gravityformsleadsquared' ),
					'type'                => 'text',
					'class'               => 'medium',
					'dependency'          => array( 'field' => 'anotherAccount', 'values' => array( '1' ) )
				),
				array(
					'name'                => 'LeadOwnerId',
					'label'               => esc_html__( 'Lead Owner Id', 'gravityformsleadsquared' ),
					'type'                => 'text',
					'class'               => 'medium',
					'dependency'          => array( 'field' => 'anotherAccount', 'values' => array( '1' ) )
				)
			)
		);

		/* Build conditional logic field array. */
		$conditional_fields = array(
			'title'      => esc_html__( 'Feed Conditional Logic', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'           => 'feedCondition',
					'type'           => 'feed_condition',
					'label'          => esc_html__( 'Conditional Logic', 'gravityformsleadsquared' ),
					'checkbox_label' => esc_html__( 'Enable', 'gravityformsleadsquared' ),
					'instructions'   => esc_html__( 'Export to LeadSquared if', 'gravityformsleadsquared' ),
					'tooltip'        => '<h6>' . esc_html__( 'Conditional Logic', 'gravityformsleadsquared' ) . '</h6>' . esc_html__( 'When conditional logic is enabled, form submissions will only be exported to LeadSquared when the condition is met. When disabled, all form submissions will be posted.', 'gravityformsleadsquared' )
				),
				
			)
		);
		
		return array( $base_fields, $lead_fields, $acvity_fields, $difac_fields  ,  $conditional_fields );

	}

	/**
	 * Set feed creation control.
	 * 
	 * @access public
	 * @return bool
	 */
	public function can_create_feed() {
		
		return $this->initialize_api();
		
	}

	/**
	 * Setup columns for feed list table.
	 * 
	 * @access public
	 * @return array
	 */
	public function feed_list_columns() {
		
		return array(
			'feedName' => esc_html__( 'Name', 'gravityformsleadsquared' ),
			'action'   => esc_html__( 'Action', 'gravityformsleadsquared' )
		);
		
	}

	/**
	 * Get value for action feed list column.
	 * 
	 * @access public
	 * @param array $feed
	 * @return string $action
	 */
	public function get_column_value_action() {
		

			
			return esc_html__( 'Create a New Lead', 'gravityformsleadsquared' );
			

		
	}





	/**
	 * Get fields for a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module (default: null)
	 * @return array $fields
	 */
	public function get_module_fields() {
		
		
		$meta_data = $this->api->get_mete_data();
		
		if ( ! empty( $meta_data ) ) {
			$fields = $meta_data;
		}
		

		return  $fields; 
		
	}

	/**
	 * Get field from a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module
	 * @param string $field_name
	 * @return array $field
	 */
	public function get_module_field( $module, $field_name ) {
		
		$module_fields = $this->get_module_fields( $module );
		
		return rgar( $module_fields, $field_name );
		
	}
	

	/**
	 * Get field map fields for a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module
	 * @param string $field_map_type (default: standard)
	 * @return array $field_map
	 */
	public function get_field_map_for_module() {
		
		$fields    = $this->get_module_fields();
		$field_map = array();
		

		
		foreach ( $fields as $field ) {

			/* Add field to field map. */
			if(!isset($field['IsReadOnly'])){
				$field_map[] = array(
					'label'      => $field['DisplayName'],
					'value'		 => $field['SchemaName']
				);
			}
		}		
		return $field_map;
		
	}

	/**
	 * Sort module fields alphabeically by label.
	 * 
	 * @access public
	 * @param array $a
	 * @param array $b
	 * @return int
	 */
	public function sort_module_fields_by_label( $a, $b ) {
		
		return strcmp( $a['label'], $b['label'] );		
		
	}

	/**
	 * Get LeadSquared users for feed field settings.
	 * 
	 * @access public
	 * @return array $users
	 */
	public function get_users_for_feed_setting() {
		
		$users = array(
			array(
				'label' => esc_html__( '-None-', 'gravityformsleadsquared' ),
				'value' => ''
			)
		);
		
		/* If API instance is not initialized, exit. */
		if ( ! $this->initialize_api() ) {
			
			$this->log_error( __METHOD__ . '(): Unable to get users because API is not initialized.' );
			return $users;
			
		}

		$lsq_users = $this->api->get_users();
		
		if ( ! empty( $lsq_users ) ) {

			foreach ( $lsq_users as $user ) {
				$name = $user['FirstName'].' '.$user['LastName'];
				$users[] = array(
					'label' => $name,
					'value' => $user['ID']	
				);
				
			}
			
		}
		
		return $users;
		
	}

	/**
	 * Process the LeadSquared feed.
	 * 
	 * @access public
	 * @param array $feed
	 * @param array $entry
	 * @param array $form
	 * @return void
	 */
	public function process_feed( $feed, $entry, $form ) {
		
		$this->log_debug( __METHOD__ . '(): Processing feed.' );
		
		/* If API instance is not initialized, exit. */
		if ( ! $this->initialize_api() ) {
		
			$this->log_error( __METHOD__ . '(): Failed to set up the API.' );
			return;
			
		}

		/* Create contact or lead */
		if ( rgars( $feed, 'meta/action' ) === 'lead' ) {			
			$lead_id = $this->create_lead( $feed, $entry, $form );		
		}

	}
	

	
	/**
	 * Create a new lead from a feed.
	 * 
	 * @access public
	 * @param array $feed
	 * @param array $entry
	 * @param array $form
	 * @return int $lead_id
	 */
	public function create_lead( $feed, $entry, $form ) {
		$replace = json_encode($entry);
		
		
		/* Create lead object. */
		/* Remove OwnerId if not set. */
		
		if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){
			if ( !rgblank( rgars( $feed, 'meta/LeadOwnerId' ) ) ) {
						$lead[] = array('Attribute' => 'OwnerId', 'Value' => rgars( $feed, 'meta/LeadOwnerId' ));
				}
		} else {
			if ( !rgblank( rgars( $feed, 'meta/OwnerId' ) ) ) {
					$lead[] = array('Attribute' => 'OwnerId', 'Value' => rgars( $feed, 'meta/OwnerId' ));
			}	
		}
		if ( !rgblank( rgars( $feed, 'meta/Source' ) ) ) {
			$lead[] = array('Attribute' => 'Source', 'Value' => rgars( $feed, 'meta/Source' ));	
		}		
		
		$mapped_fields  = $this->api->get_dynamic_field_map_fields_lsq( $feed);
		$count = 0;
		
		foreach ($form['fields'] as $field) {
			if($field->type == 'survey') {		
				$count2 = 0;
				foreach ((array) $field->inputs as $input) {	
					foreach ($input as $key => $value) {
						$name = $form['fields'][$count]->inputs[$count2]['name'];
						$label = $form['fields'][$count]->inputs[$count2]['label'] ;	
						$replace = str_replace($name, $label,$replace);
					}
				$count2++;
				}
				$countc = 0;
				foreach ((array) $field->choices as $input) {	
					foreach ($input as $key => $value) {
						$value = $form['fields'][$count]->choices[$countc]['value'];
						$text = $form['fields'][$count]->choices[$countc]['text'];
						$replace = str_replace($value, $text,$replace);
					}
				$countc++;
				}				
			}			
			$count++;
		}
		$entry =  json_decode($replace,true);
		
		foreach ($form['fields'] as $field) {
			if($field['type'] == 'multiselect'){
				$multiselect = $field['id'];
			}
			
		}
		
		foreach ( $mapped_fields as $field_name => $field_id ) {
			
			$field_value = $this->process_field_value_lsq($form, $entry, $value, $field_id);
			if ( rgblank( $field_value ) )
				continue;
			
			$atti = array('Attribute' => $field_name, 'Value' => $field_value );	
			
			$lead[] = $atti;
			
		}
		
		if(!in_array('ProspectID', array_column($lead, 'Attribute'))) { 
			if(!in_array('Source', array_column($lead, 'Attribute'))){
				$atti = array('Attribute' => 'Source', 'Value' => 'GravityForm');	
				$lead[] = $atti;
			}
		}
		/* Filter lead. */
		$lead = gf_apply_filters( 'gform_leadsquared_lead', $form['id'], $lead, $feed, $entry, $form );
	

		
		$this->log_debug( __METHOD__ . '(): Creating lead: ' . print_r( $lead, true ) );
		if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){	
			$akey = rgars( $feed, 'meta/AccessKey' );
			$skey = rgars( $feed, 'meta/SecretKey' );
		} 
		
		try {
			/* Insert lead record. */

			if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){
				$lead_record = $this->api->insert_record($lead,$akey,$skey);
			} else {
				$lead_record = $this->api->insert_record($lead);					
			}

			if( is_wp_error( $lead_record ) ) {
				$error_log = json_encode($lead_record);
				
				$json_data = json_encode($lead,true);
				gform_update_meta( $entry['id'], 'leadsquared_json_data', $json_data);
					
				gform_update_meta( $entry['id'], 'leadsquared_error_type', $error_log);
					
				gform_update_meta( $entry['id'], 'leadsquared_error_message', "Error due to timed out or Could not resolve host");	
				
				$to = array('support@leadsquared.com',get_option( 'admin_email' ));
				$site_url = get_site_url();
				$subject = 'Gravity Form Error for '.$site_url;
				$body = '<html> <head> <meta name="viewport" content="width=device-width"> <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"> </head> <body><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:#ececec;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="600" style="width:6.25in;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="600" style="width:6.25in;border-collapse:collapse"><tbody><tr><td style="padding:11.25pt 11.25pt 11.25pt 11.25pt"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:18.75pt 0in 11.25pt 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td valign="top" style="padding:0in 0in 0in 0in"><p class="MsoNormal"><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:black"><a href="http://www.leadsquared.com" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFxE_S3GID2h2CS9aAztSr1sElzyQ"><b><img border="0" id="m_-2092094941101708403_x0000_i1025" src="https://ci3.googleusercontent.com/proxy/HjpS7yd-s_cmeyLj2wdQQ_aqMG4UTmvO3yo2cTHqm8x8G3iPCG3LuEpYzFlbDOAZngmnMrxmRBfs8gW6Qi_mO5FuFNcxdofOKhzRRUFsQsbcEarA3tKm1DaCul4L1TKEvaJ83T5S0ZEkJw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-logo.png" alt="Leadsquared" class="CToWUd"></b></a><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div><p class="MsoNormal" align="center" style="text-align:center"><u></u>&nbsp;<u></u></p><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:white;border-collapse:collapse;border-radius:5px"><tbody><tr><td style="padding:15.0pt 7.5pt 15.0pt 7.5pt;font-size:9pt!important;color:#343434!important"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse;font-size:9pt!important;color:#343434!important"><tbody><tr><td style="padding:0in 0in 0in 0in"><p class="MsoNormal"><span style="font-size:10.0pt;font-family:&quot;Arial&quot;,sans-serif;color:#343434">Hi there,<br><br>There was an error while processing lead from Gravity Form.<br><br>Error: '.$error_log.'<br><br>Json: '.$json_data.'<br><br>Cheers!<br>Team LeadSquared<br><br>Please do not respond to this email.</span><span style="font-family:&quot;Arial&quot;,sans-serif"><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div><p class="MsoNormal"><span style="display:none"><u></u>&nbsp;<u></u></span></p><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:#ececec;border-collapse:collapse;border-spacing:0"><tbody><tr><td style="padding:11.25pt 15.0pt 11.25pt 15.0pt;font-size:8pt!important;line-height:12px!important;color:#454545!important;line-height:18px!important"><p class="MsoNormal" align="center" style="text-align:center"><span style="font-size:8.0pt;font-family:&quot;Helvetica&quot;,sans-serif;color:#454545">©&nbsp;Copyright by<a href="http://www.leadsquared.com" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFxE_S3GID2h2CS9aAztSr1sElzyQ">LeadSquared</a>. All Rights Reserved.<br><a href="http://www.leadsquared.com/company" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com/company&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFEnG6__sRwk03lB80e1ab_IGzCIw">About Us</a> &nbsp;|&nbsp; <a href="http://www.leadsquared.com/company/contact" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com/company/contact&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNHH5bvMLiJz57Ndg0REO6Xv1AVM1w">Contact</a></span><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:#454545"><u></u><u></u></span></p></td></tr><tr><td style="padding:0in 0in 0in 0in"><p class="MsoNormal" align="center" style="text-align:center"><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:#454545"><a href="https://twitter.com/LeadSquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://twitter.com/LeadSquared&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNGEmuUkevLoSG9l1WOXWzQ5BbbNlA"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1026" src="https://ci5.googleusercontent.com/proxy/8KWVjCfC4DhNDUsjOfTBCbByLUw6Us4Vs3MpG1lirV6aK1b4F3zz1kEeS0byBi6uamg0PCximOwQSWFY1xz3pUa18iK-6_tPS9uvwFWFQU5zXW6_1oCImUBz5SZclun7nwZXEGmnIdK9MhwDaw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-twitter.png" class="CToWUd"></a>&nbsp;&nbsp;<a href="https://www.facebook.com/LeadSquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://www.facebook.com/LeadSquared&amp;source=gmail&amp;ust=1480051784394000&amp;usg=AFQjCNFlf4-kqlcaaaPJD3l3ylWFEPdnGg"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1027" src="https://ci4.googleusercontent.com/proxy/DCmzas-VcDKTyuejq8KHdpcGvWEVrAoE9YUZYIfiLPVIt_YUROBpGB9-TMaGBC4hr3aXD0TULtn3mi-fS4-BeZV8pw99ir6SBY5DrFqZGj9CpHycnc0V85XdhnO985FAbc3K-_try1RMnj6ynRk=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-facebook.png" class="CToWUd"></a>&nbsp;&nbsp;<a href="https://www.linkedin.com/company/leadsquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://www.linkedin.com/company/leadsquared&amp;source=gmail&amp;ust=1480051784394000&amp;usg=AFQjCNELgOXeYS_VkHMslgLT0NZmVCHfWw"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1028" src="https://ci5.googleusercontent.com/proxy/b3ZmKT-HJzECJVaADrLcLwSg8xeFBdVCCIe-GGrQAO6KMCP_lHadYAuXY2moYhRGeCk7p1bo9moQ6gE1POPdEywEnlsur5AuadjyJbK1rH3z4RR3WBdPZXuK76PeT0Sjm4AaWqkeb1EYjg0o0dw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-linkedin.png" class="CToWUd"></a><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></body></html>';
				$headers = array('Content-Type: text/html; charset=UTF-8');
				 
				wp_mail( $to, $subject, $body, $headers );
				
				return null;
			} else {
				if($lead_record['Status'] == 'Success'){
					$json_data = json_encode($lead,true);
					gform_update_meta( $entry['id'], 'leadsquared_json_data', $json_data );

					$lead_id = $lead_record['Message']['RelatedId'];
					$activity_id = $this->post_Activity( $lead_id , $feed, $entry, $form ); 
					 

					/* Save lead ID to entry meta. */
					gform_update_meta( $entry['id'], 'leadsquared_lead_id', $lead_id );
					if($activity_id){
						gform_update_meta( $entry['id'], 'leadsquared_activity_id', $activity_id );
					}
					
					/* Log that lead was created. */
					$this->log_debug( __METHOD__ . '(): Lead #' . $lead_id . ' created.' );
					
					return $lead_id;
				} else {
					
					$json_data = json_encode($lead,true);
					gform_update_meta( $entry['id'], 'leadsquared_json_data', $json_data );
					
					$error_type = $lead_record['ExceptionType'];
					gform_update_meta( $entry['id'], 'leadsquared_error_type', $error_type );
					
					$error_message = $lead_record['ExceptionMessage'];
					gform_update_meta( $entry['id'], 'leadsquared_error_message', $error_message );	
					
					$this->log_error( __METHOD__ . '(): Could not create Lead; ' . $lead_record['ExceptionMessage'] );
					
					$error_log = json_encode($lead_record);
					$json_data = json_encode($lead,true);
					
					$to = array(get_option( 'admin_email' ));
					$site_url = get_site_url();
					$subject = 'Gravity Form Error for '.$site_url;
					$body = '<html> <head> <meta name="viewport" content="width=device-width"> <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"> </head> <body><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:#ececec;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="600" style="width:6.25in;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="600" style="width:6.25in;border-collapse:collapse"><tbody><tr><td style="padding:11.25pt 11.25pt 11.25pt 11.25pt"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:18.75pt 0in 11.25pt 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td style="padding:0in 0in 0in 0in"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse"><tbody><tr><td valign="top" style="padding:0in 0in 0in 0in"><p class="MsoNormal"><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:black"><a href="http://www.leadsquared.com" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFxE_S3GID2h2CS9aAztSr1sElzyQ"><b><img border="0" id="m_-2092094941101708403_x0000_i1025" src="https://ci3.googleusercontent.com/proxy/HjpS7yd-s_cmeyLj2wdQQ_aqMG4UTmvO3yo2cTHqm8x8G3iPCG3LuEpYzFlbDOAZngmnMrxmRBfs8gW6Qi_mO5FuFNcxdofOKhzRRUFsQsbcEarA3tKm1DaCul4L1TKEvaJ83T5S0ZEkJw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-logo.png" alt="Leadsquared" class="CToWUd"></b></a><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div><p class="MsoNormal" align="center" style="text-align:center"><u></u>&nbsp;<u></u></p><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:white;border-collapse:collapse;border-radius:5px"><tbody><tr><td style="padding:15.0pt 7.5pt 15.0pt 7.5pt;font-size:9pt!important;color:#343434!important"><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;border-collapse:collapse;font-size:9pt!important;color:#343434!important"><tbody><tr><td style="padding:0in 0in 0in 0in"><p class="MsoNormal"><span style="font-size:10.0pt;font-family:&quot;Arial&quot;,sans-serif;color:#343434">Hi there,<br><br>There was an error while processing lead from Gravity Form.<br><br>Error: '.$error_log.'<br><br>Json: '.$json_data.'<br><br>Cheers!<br>Team LeadSquared<br><br>Please do not respond to this email.</span><span style="font-family:&quot;Arial&quot;,sans-serif"><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div><p class="MsoNormal"><span style="display:none"><u></u>&nbsp;<u></u></span></p><div align="center"><table class="m_-2092094941101708403MsoNormalTable" border="0" cellspacing="0" cellpadding="0" width="100%" style="width:100.0%;background:#ececec;border-collapse:collapse;border-spacing:0"><tbody><tr><td style="padding:11.25pt 15.0pt 11.25pt 15.0pt;font-size:8pt!important;line-height:12px!important;color:#454545!important;line-height:18px!important"><p class="MsoNormal" align="center" style="text-align:center"><span style="font-size:8.0pt;font-family:&quot;Helvetica&quot;,sans-serif;color:#454545">©&nbsp;Copyright by<a href="http://www.leadsquared.com" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFxE_S3GID2h2CS9aAztSr1sElzyQ">LeadSquared</a>. All Rights Reserved.<br><a href="http://www.leadsquared.com/company" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com/company&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNFEnG6__sRwk03lB80e1ab_IGzCIw">About Us</a> &nbsp;|&nbsp; <a href="http://www.leadsquared.com/company/contact" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=http://www.leadsquared.com/company/contact&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNHH5bvMLiJz57Ndg0REO6Xv1AVM1w">Contact</a></span><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:#454545"><u></u><u></u></span></p></td></tr><tr><td style="padding:0in 0in 0in 0in"><p class="MsoNormal" align="center" style="text-align:center"><span style="font-family:&quot;Helvetica&quot;,sans-serif;color:#454545"><a href="https://twitter.com/LeadSquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://twitter.com/LeadSquared&amp;source=gmail&amp;ust=1480051784393000&amp;usg=AFQjCNGEmuUkevLoSG9l1WOXWzQ5BbbNlA"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1026" src="https://ci5.googleusercontent.com/proxy/8KWVjCfC4DhNDUsjOfTBCbByLUw6Us4Vs3MpG1lirV6aK1b4F3zz1kEeS0byBi6uamg0PCximOwQSWFY1xz3pUa18iK-6_tPS9uvwFWFQU5zXW6_1oCImUBz5SZclun7nwZXEGmnIdK9MhwDaw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-twitter.png" class="CToWUd"></a>&nbsp;&nbsp;<a href="https://www.facebook.com/LeadSquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://www.facebook.com/LeadSquared&amp;source=gmail&amp;ust=1480051784394000&amp;usg=AFQjCNFlf4-kqlcaaaPJD3l3ylWFEPdnGg"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1027" src="https://ci4.googleusercontent.com/proxy/DCmzas-VcDKTyuejq8KHdpcGvWEVrAoE9YUZYIfiLPVIt_YUROBpGB9-TMaGBC4hr3aXD0TULtn3mi-fS4-BeZV8pw99ir6SBY5DrFqZGj9CpHycnc0V85XdhnO985FAbc3K-_try1RMnj6ynRk=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-facebook.png" class="CToWUd"></a>&nbsp;&nbsp;<a href="https://www.linkedin.com/company/leadsquared" target="_blank" data-saferedirecturl="https://www.google.com/url?hl=en&amp;q=https://www.linkedin.com/company/leadsquared&amp;source=gmail&amp;ust=1480051784394000&amp;usg=AFQjCNELgOXeYS_VkHMslgLT0NZmVCHfWw"><img border="0" width="24" height="24" style="width:.25in;height:.25in" id="m_-2092094941101708403_x0000_i1028" src="https://ci5.googleusercontent.com/proxy/b3ZmKT-HJzECJVaADrLcLwSg8xeFBdVCCIe-GGrQAO6KMCP_lHadYAuXY2moYhRGeCk7p1bo9moQ6gE1POPdEywEnlsur5AuadjyJbK1rH3z4RR3WBdPZXuK76PeT0Sjm4AaWqkeb1EYjg0o0dw=s0-d-e1-ft#https://d3mzbdkbfcvx77.cloudfront.net/leadsquared/common/images/leadsquared-linkedin.png" class="CToWUd"></a><u></u><u></u></span></p></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></div></td></tr></tbody></table></body></html>';
					$headers = array('Content-Type: text/html; charset=UTF-8');
				 
					wp_mail( $to, $subject, $body, $headers );
					return null;
				}
			}
		} catch ( Exception $e ) {
			
			$this->log_error( __METHOD__ . '(): Could not create Lead; ' . $e->getMessage() );
			
			return null;
			
		}
		
	}
	
	public function process_field_value_lsq($form, $entry, $value, $field_id){
		if(is_array($field_id)){
				foreach($field_id as $value){				
					$field_val[] = $this->api->get_field_value( $form, $entry, $value );
				}
				$field_val = array_filter($field_val, function($value) { return $value !== ''; });
				$field_value = implode(',',$field_val);	
				$field_value = ltrim($field_value,",");
				$field_value = rtrim($field_value,",");			
				return $field_value;
			} else {
				$field_value = $this->get_field_value( $form, $entry, $field_id );
				if($field_id == $multiselect){
					$field_value = str_replace(",",";",$field_value);
				}
				
				return $field_value;
			}
			
	}
	
	public function post_Activity( $lead_id , $feed, $entry, $form ) {
			
			if ( rgars( $feed, 'meta/postActivity' ) != '1' )
				return;
			
			
			if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){
					$AENumber = rgars( $feed, 'meta/ActivityEventCode' );
				} else {
					$AENumber = rgars( $feed, 'meta/ActivityEvent' );				
			}
			$activityDetails = array(
				'RelatedProspectId'	=> $lead_id,
				'ActivityEvent'	=>	$AENumber,
				'ActivityNote'	=>	GFCommon::replace_variables( $feed['meta']['ActivityNote'], $form, $entry, false, false, false, 'text' ),
			);
			
			if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){	
				$akey = rgars( $feed, 'meta/AccessKey' );
				$skey = rgars( $feed, 'meta/SecretKey' );
			} 
			try {
			
				/* Insert task record. */
				if ( rgars( $feed, 'meta/anotherAccount' ) == '1' ){
					$activity_record = $this->api->insert_activity($activityDetails,$akey,$skey);
				} else {
					$activity_record = $this->api->insert_activity($activityDetails);					
				}
				return $activity_record['Message']['Id'];
			
			} catch ( Exception $e ) {
				
				$this->log_error( __METHOD__ . '(): Could not create task; ' . $e->getMessage() );
				
				return null;
				
			}
			
		}
	/**
	 * Initialized LeadSquared API if credentials are valid.
	 * 
	 * @access public
	 * @return bool
	 */
	public function initialize_api() {

		if ( ! is_null( $this->api ) ) {
			return true;
		}
		
		/* Include the API library. */
		if ( ! class_exists( 'LSQ_GF' ) ) {
			require_once 'includes/class-leadsquared.php';
		}

		/* Get the plugin settings */
		$settings = $this->get_plugin_settings();
		

		/* If any of the account information fields are empty, return null. */
		if ( rgblank( $settings['AccessKey'] ) || rgblank( $settings['SecretKey'] )|| rgblank( $settings['apidomain'] ) )
			return null;
			
		$this->log_debug( __METHOD__ . "(): Validating API credentials." );
		
		$leadsquared = new LSQ_GF( $settings['AccessKey'],$settings['SecretKey'],$settings['apidomain'] );
		
		try {
			
			/* Run API test. */
			if($leadsquared->user_authentication() == true){
				/* Log that test passed. */
				$this->log_debug( __METHOD__ . '(): API credentials are valid.' );
				
				/* Assign LeadSquared object to the class. */
				$this->api = $leadsquared;
				
				return true;
			} else {
				return false;
			}
			
			
		} catch ( Exception $e ) {
			
			/* Log that test failed. */
			$this->log_error( __METHOD__ . '(): API credentials are invalid; '. $e->getMessage() );			

			return false;
			
		}
		
	}

}
